#ifndef __APCIE1564_KNOWNDEV_H__
#define __APCIE1564_KNOWNDEV_H__

void apcie1564_known_dev_append(struct pci_dev * pdev);
void apcie1564_known_dev_remove(struct pci_dev * pdev);

#endif // __APCIE1564_KNOWNDEV_H__
